;===============================================================================
; Nested modeless dialog demo, modeless dialog as main.
;
; The CallModelessDialog macro is included in MASM32 v10 only.
;===============================================================================
    include \masm32\include\masm32rt.inc
;===============================================================================

loword MACRO arg
    IFNDEF __loword_macro_return_var__
      .data
        __loword_macro_return_var__ dd 0
      .code
    ENDIF
    push arg
    pop __loword_macro_return_var__
    and __loword_macro_return_var__, 0ffffh
    EXITM <__loword_macro_return_var__>
ENDM

hiword MACRO arg
    IFNDEF __hiword_macro_return_var__
      .data
        __hiword_macro_return_var__ dd 0
      .code
    ENDIF
    push arg
    pop __hiword_macro_return_var__
    shr __hiword_macro_return_var__, 16
    EXITM <__hiword_macro_return_var__>
ENDM

IDC_BTNGO equ 101

;===============================================================================
    .data
      hDlgMain   HWND 0
      hDlgNested HWND 0
      msg        MSG  <>
    .code
;===============================================================================

NestedDialogProc proc hwndDlg:HWND, uMsg:UINT, wParam:WPARAM, lParam:LPARAM

    SWITCH uMsg

      CASE WM_COMMAND

        SWITCH loword(wParam)

          CASE IDCANCEL

            invoke DestroyWindow, hwndDlg

            ;--------------------------------------------------------
            ; Zero the nested dialog window handle so MainDialogProc
            ; will be able to determine that the dialog is not open.
            ;--------------------------------------------------------

            mov hDlgNested, 0

        ENDSW

      CASE WM_CLOSE

        invoke DestroyWindow, hwndDlg

        ;--------------------------------------------------------
        ; Zero the nested dialog window handle so MainDialogProc
        ; will be able to determine that the dialog is not open.
        ;--------------------------------------------------------

        mov hDlgNested, 0

      ;--------------------------------------------------------
      ; Because both dialogs share the message loop, to ensure
      ; that closing this dialog will not terminate the app,
      ; we must avoid handling the WM_DESTROY message and
      ; calling PostQuitMessage, as is done in MainDialogProc.
      ;--------------------------------------------------------

    ENDSW

    xor eax, eax
    ret

NestedDialogProc endp

;===============================================================================

MainDialogProc proc hwndDlg:HWND, uMsg:UINT, wParam:WPARAM, lParam:LPARAM

    SWITCH uMsg

      CASE WM_COMMAND

        SWITCH loword(wParam)

          CASE IDC_BTNGO

            ;------------------------------------------------------
            ; Because there is only a single variable to store the
            ; window handle for the nested dialog, and because a
            ; valid window handle is required for the message loop
            ; to process messages for a dialog, we need to prevent
            ; the user from opening more than one instance of the
            ; nested dialog. If the nested dialog is open, then
            ; shift the input focus to it; otherwise create a new
            ; nested dialog.
            ;------------------------------------------------------

            .IF hDlgNested

              invoke SetFocus, hDlgNested

            .ELSE

              Dialog  "NestedDialog", \             ; caption
              "MS Sans Serif", 10, \                ; font,pointsize
              WS_OVERLAPPED or WS_SYSMENU or \      ; window style
              WS_VISIBLE or DS_CENTER, \
              1, \                                  ; number of controls
              0,0,110,90, \                         ; x, y, width, height
              1024                                  ; memory buffer size

              DlgButton "Close",0,35,50,40,12,IDCANCEL

              invoke GetModuleHandle, NULL
              CallModelessDialog eax, 0, NestedDialogProc, NULL
              mov hDlgNested, eax

            .ENDIF

          CASE IDCANCEL

            invoke DestroyWindow, hwndDlg

        ENDSW

      CASE WM_CLOSE

        invoke DestroyWindow, hwndDlg

      CASE WM_DESTROY

        invoke PostQuitMessage, NULL

    ENDSW

    xor eax, eax
    ret

MainDialogProc endp

;===============================================================================
start:
;===============================================================================

    Dialog  "Main Dialog", \                  ; caption
            "MS Sans Serif", 10, \            ; font,pointsize
            WS_OVERLAPPED or WS_SYSMENU or \  ; window style
            WS_VISIBLE or DS_CENTER, \
            2, \                              ; number of controls
            0,0,150,100, \                    ; x, y, width, height
            1024                              ; memory buffer size

    DlgButton "Go",0,30,60,40,12,IDC_BTNGO

    DlgButton "Close",0,80,60,40,12,IDCANCEL

    invoke GetModuleHandle, NULL
    CallModelessDialog eax, 0, MainDialogProc, NULL
    mov hDlgMain, eax

  msgLoop:

    invoke GetMessage, ADDR msg, 0, 0, 0
    .IF eax != 0
      invoke IsDialogMessage, hDlgMain, ADDR msg
      .IF eax == 0
        invoke IsDialogMessage, hDlgNested, ADDR msg
      .ENDIF
      jmp msgLoop
    .ENDIF

    invoke ExitProcess, eax

;===============================================================================
end start
